import obj_player
import math

class Bullet:
    '''This is the bullet template and won't be used in game except for actual bullets to inherit from'''
    def __init__(self,x,y,d):
        '''
        x = x coordinate of bullet
        y = y coordiante of bullet
        d = direction of travel of bullet (angle, degrees)
        '''
        self.x = float(x)
        self.y = float(y)
        self.d = int(d)
        self.lifetime = 0
        self.health = 1 #bullets don't have health in the traditional sense but we can remove them by setting health = 0
        self.output = []
        self.miss = 0
        self.hitbox = 5 #radius of the hitbox
        self.base_score = 0
    
    def update(self):
        self.lifetime += 1

    def check_mortality(self):
        '''This function: 
        - destroys the bullet if it goes far out of bounds
        - destroys the player if they go inside the bullet hitbox
        '''
        if self.x < -50 or self.x > 650 or self.y < -50 or self.y > 450:
            self.health = 0
        if (obj_player.x - self.x)**2 + (obj_player.y - self.y)**2 < (self.hitbox)**2:
            self.miss = 1

class Straight(Bullet):
    '''The basic bullet, this will move in a straight line in the direction defined at a constant speed'''
    def __init__(self,x,y,d,speed):
        Bullet.__init__(self,x,y,d)
        self.speed = speed

    def update(self):
        '''The angle is defined at creation, and moves in a straight line by calculating change in x and y'''
        self.lifetime += 1
        self.x += float(self.speed) * math.cos(math.radians(self.d))
        self.y += float(self.speed) * math.sin(math.radians(self.d))
        self.check_mortality() #this goes at the end which so that the state that the player sees is checked

class DelayHoming(Bullet):
    def __init__(self, x, y, d,speed):
        super().__init__(x, y, d)
        self.speed = speed
        self.hitbox = 10

    def update(self):
        self.lifetime += 1
        if self.lifetime < 50 or self.lifetime > 60:
            self.x += float(self.speed) * math.cos(math.radians(self.d))
            self.y += float(self.speed) * math.sin(math.radians(self.d))
        if self.lifetime == 60:
            self.d = math.degrees(math.atan2(obj_player.y-self.y,obj_player.x-self.x))
            self.speed = str(float(self.speed) * 1.5)
        self.check_mortality()